unit hdObjs;

//TODO: Free TStringLists

// Copyright (c) 1999 by Ziff-Davis, Inc.
// Written by Neil J. Rubenking

interface
USES Classes, SysUtils, Windows, shlobj, graphics, activex, comctrls;

const
  cuRecycle = 0;
  cuHolding = 1;
  cuDelete  = 2;
  IgnoreSubst : Boolean = FALSE; //1.01
type

TJReptType = procedure(vSizDel : Int64; vNumDir, vNumDel, vNumRes,
  vNumProt : Integer) OF Object;
TJDoFilType = procedure(const S : String; fSize : Int64) OF Object;

TJunkFileObj = class(TObject)
  private
    fName                  : String;
    fChecked               : Boolean;
    fBacktype              : Integer;
    fDirs                  : TStringList; // folders to search
    fDeletes               : TStringList; // filespecs to delete
    fProts                 : TStringList; // filespecs to protect
    // Date restrictions
    fDateRestrict          : Boolean;
    fDateRestrictType      : Integer;
    fDateData1, fDateData2 : TDateTime;
    // Age restrictions
    fAgeRestrict           : Boolean;
    fAgeRestrictType       : Integer;
    fAgeData1, fAgeData2   : Integer;
    // Size restrictions
    fSizeRestrict          : Boolean;
    fSizeRestrictType      : Integer;
    fSizeData1, fSizeData2 : Integer;
    fSizeUnit              : Integer;
    procedure SetBackType(const Value: Integer);
    procedure SetDateRestrictType(const Value: Integer);
    procedure SetDeletes(const Value: TStringList);
    procedure SetDirs(const Value: TStringList);
    procedure SetSizeRestrictType(const Value: Integer);
    procedure SetSizeUnit(const Value: Integer);
    procedure SetProts(const Value: TStringList);
    procedure SetAgeRestrictType(const Value: Integer);
protected
public
  constructor Create; overload;
  constructor Create(TJ : TJunkFileObj); overload;
  constructor Create(const vName : String; vChecked : Boolean;
    vBacktype : Integer; vDirs, vDeletes, vProts : TStrings;
    vDateRestrict : Boolean; vDateRestrictType : Integer; vDateData1,
    vDateData2 : TDateTime; vAgeRestrict : Boolean; vAgeRestrictType,
    vAgeData1, vAgeData2 : Integer; vSizeRestrict : Boolean;
    vSizeRestrictType, vSizeData1, vSizeData2, vSizeUnit : Integer);
    overload;
  constructor Create(const vName: String; vChecked: Boolean;
    vBacktype: Integer; vDirs, vDeletes, vProts: String;
    Onlies : DWORD); overload;
  function  SameAs(TJ : TJunkFileObj) : Boolean;
  procedure SaveToStream(S : TStream);
  procedure LoadFromStream(S : TStream);
  procedure REDisplay(vRE : TRichEdit);
  procedure ActOnFiles(DoWhat: TJDoFilType; DoReport: TJReptType;
    WhenDone : TNotifyEvent; vStopEvent : THandle;
    const vMyDir : String);

  function MatchProts(const sD, sN : String) : Boolean;
  function MatchRestrict(TD : TWin32FindData) : Boolean;
  property TypName : String read fName write fName;
  property IsChecked : Boolean read fChecked write fChecked;
  property Backtype : Integer read fBackType write SetBackType;
  property Dirs : TStringList read fDirs write SetDirs;
  property Deletes : TStringList read fDeletes write SetDeletes;
  property Prots : TStringList read fProts write SetProts;
  property DateRestrict : Boolean read fDateRestrict
                                  write fDateRestrict;
  property DateRestrictType : Integer read fDateRestrictType
                                      write SetDateRestrictType;
  property DateData1 : TDateTime read fDateData1;
  property DateData2 : TDateTime read fDateData2;
  property AgeRestrict : Boolean read fAgeRestrict
                                 write fAgeRestrict;
  property AgeRestrictType : Integer read fAgeRestrictType
                                     write SetAgeRestrictType;
  property AgeData1 : Integer read fAgeData1;
  property AgeData2 : Integer read fAgeData2;
  property SizeRestrict : Boolean read fSizeRestrict
                                  write fSizeRestrict;
  property SizeRestrictType : Integer read fSizeRestrictType
                                      write SetSizeRestrictType;
  property SizeData1 : Integer read fSizeData1;
  property SizeData2 : Integer read fSizeData2;
  property SizeUnit : Integer read fSizeUnit write SetSizeUnit;
end;

TJunkDelThread = class(TThread)
private
  fNumDir  : Integer;
  fNumDel  : Integer;
  fNumRes  : Integer;
  fNumProt : Integer;
  fSizDel  : Int64;
  fMyDir   : String;
protected
  fDoWhat    : TJDoFilType;
  fReport    : TJReptType;
  fTJ        : TJunkFileObj;
  fTmp1      : String;
  fTmpSiz    : Int64;
  fStopEvent : THandle;
  fStopped   : Boolean;
public
  constructor Create(vTJ : TJunkFileObj; vDoWhat : TJDoFilType;
    vReport : TJReptType; vWhenDone : TNotifyEvent;
    vStopEvent : THandle; const vMyDir : String);
  procedure Execute; override;
  procedure SyncDoWhat;
  procedure SyncReport;
  property WasStopped : Boolean read fStopped;
end;

procedure AddAllDefaults(LV : TListView; RecycleProb : Boolean);

implementation

uses AllFuncs, pmatchu;
{ TJunkFileObj }

function NameForAll : String;
const TheName : String = '';
begin
  IF TheName = '' THEN
    TheName := Format('Local drives (%s)', [LocalDrives(':',',',
      IgnoreSubst)]); //1.01
  Result := TheName;
end;

function IECache : String;
VAR
  pidl : pItemIDList;
  Buffer : ARRAY[0..MAX_PATH] OF Char;
begin
  SHGetSpecialFolderLocation(0, CSIDL_INTERNET_CACHE, pIdl);
  SHGetPathfromIDList(pIDL, Buffer);
  CotaskMemFree(pIDL);
  Result := StrPas(Buffer);
  IF Length(Result) < 4 THEN Result := ''; //1.01
end;

constructor TJunkFileObj.Create;
begin
  fName               := '';
  fChecked            := True;
  fBacktype           := cuRecycle;
  fDirs               := TStringList.Create;
  fDeletes            := TStringList.Create;
  fProts              := TStringList.Create;
  fDateRestrict       := FALSE;
  fDateRestrictType   := 0;
  fDateData1          := 0.0;
  fDateData2          := 0.0;
  fAgeRestrict        := False;
  fAgeRestrictType    := 0;
  fAgeData1           := 0;
  fAgeData2           := 0;
  fSizeRestrict       := False;
  fSizeRestrictType   := 0;
  fSizeData1          := 0;
  fSizeData2          := 0;
  fSizeUnit           := 0;
end;

constructor TJunkFileObj.Create(const vName : String; vChecked :
  Boolean; vBacktype : Integer; vDirs, vDeletes, vProts : TStrings;
  vDateRestrict : Boolean; vDateRestrictType : Integer; vDateData1,
  vDateData2 : TDateTime; vAgeRestrict : Boolean; vAgeRestrictType,
  vAgeData1, vAgeData2 : Integer; vSizeRestrict : Boolean;
  vSizeRestrictType, vSizeData1, vSizeData2, vSizeUnit : Integer);
begin
  fName      := vName;
  fChecked   := vChecked;
  fBacktype  := vBacktype;
  fDirs      := TStringList.Create;
  fDirs.Assign(vDirs);
  fDeletes   := TStringList.Create;
  fDeletes.Assign(vDeletes);
  fProts     := TStringList.Create;
  fProts.Assign(vProts);
  fDateRestrict     := vDateRestrict;
  fDateRestrictType := vDateRestrictType;
  fDateData1        := Int(vDateData1);
  IF (vDateRestrictType > 2) AND
    (Int(vDateData2) < Int(vDateData1)) THEN
    begin
      fDateData2    := Int(vDateData1);
      fDateData1    := Int(vDateData2);
    end
  ELSE
    fDateData2      := Int(vDateData2);
  fAgeRestrict      := vAgeRestrict;
  fAgeRestrictType  := vAgeRestrictType;
  fAgeData1         := vAgeData1;
  IF (vAgeRestrictType > 2) AND (vAgeData2 < vAgeData1) THEN
    begin
      fAgeData1     := vAgeData2;
      fAgeData2     := vAgeData1;
    end
  ELSE
    fAgeData2       := vAgeData2;
  fSizeRestrict     := vSizeRestrict;
  fSizeRestrictType := vSizeRestrictType;
  fSizeData1        := vSizeData1;
  IF (vSizeRestrictType > 2) AND (vSizeData2 < vSizeData1) THEN
    begin
      fSizeData1    := vSizeData2;
      fSizeData2    := vSizeData1;
    end
  ELSE
    fSizeData2      := vSizeData2;
  fSizeUnit         := vSizeUnit;
end;

procedure TJunkFileObj.SaveToStream(S: TStream);
VAR
  Writer : TWriter;
  N      : Integer;
begin
  Writer := TWriter.Create(S, 4096);
  WITH Writer DO
  try
    WriteString(fName);
    WriteBoolean(fChecked);
    WriteInteger(fBacktype);
    WriteInteger(fDirs.Count);
    FOR N := 0 TO fDirs.Count-1 DO
      begin
        WriteString(fDirs[N]);
        WriteInteger(Integer(fDirs.Objects[N]));
      end;
    WriteInteger(fDeletes.Count);
    FOR N := 0 TO fDeletes.Count-1 DO
      WriteString(fDeletes[N]);
    WriteInteger(fProts.Count);  
    FOR N := 0 TO fProts.Count-1 DO
      WriteString(fProts[N]);
    WriteBoolean(fDateRestrict);
    WriteInteger(fDateRestrictType);
    WriteDate(fDateData1);
    WriteDate(fDateData2);
    WriteBoolean(fAgeRestrict);
    WriteInteger(fAgeRestrictType);
    WriteInteger(fAgeData1);
    WriteInteger(fAgeData2);
    WriteBoolean(fSizeRestrict);
    WriteInteger(fSizeRestrictType);
    WriteInteger(fSizeData1);
    WriteInteger(fSizeData2);
    WriteInteger(fSizeUnit);
  finally
    Free;
  end;
end;

procedure TJunkFileObj.LoadFromStream(S: TStream);
VAR
  Reader : TReader;
  N, Count      : Integer;
begin
  Reader := TReader.Create(S, 4096);
  WITH Reader DO
  try
    fName := ReadString();
    fChecked := ReadBoolean;
    fBacktype := ReadInteger();
    Count := ReadInteger();
    FOR N := 0 TO Count-1 DO
      fDirs.AddObject(ReadString, Pointer(ReadInteger));
    Count := ReadInteger();
    FOR N := 0 TO Count-1 DO
      fDeletes.Add(ReadString);
    Count := ReadInteger;
    FOR N := 0 TO Count-1 DO
      fProts.Add(ReadString);
    fDateRestrict :=     ReadBoolean;
    fDateRestrictType := ReadInteger;
    fDateData1 :=        ReadDate;
    fDateData2 :=        ReadDate;
    fAgeRestrict :=      ReadBoolean;
    fAgeRestrictType :=  ReadInteger;
    fAgeData1 :=         ReadInteger;
    fAgeData2 :=         ReadInteger;
    fSizeRestrict :=     ReadBoolean;
    fSizeRestrictType := ReadInteger;
    fSizeData1 :=        ReadInteger;
    fSizeData2 :=        ReadInteger;
    fSizeUnit :=         ReadInteger;
  finally
    Free;
  end;
end;

procedure TJunkFileobj.REDisplay(vRE : TRichEdit);
CONST
  BB : TFontStyles = [fsBold];
  UU : TFontStyles = [fsUnderline];
  II : TFontStyles = [fsItalic];
  BU : TFontStyles = [fsBold, fsUnderline];
  BI : TFontStyles = [fsBold, fsItalic];
  UI : TFontStyles = [fsUnderline, fsItalic];
  xx : TFontStyles = [];
  ard1 : ARRAY[0..4] OF String = ('exactly', 'before', 'after',
    'between', 'not between');
  ard2 : ARRAY[0..4] OF String = ('exactly', 'older than',
    'newer than', 'between', 'not between');
  ard3 : ARRAY[0..4] OF String = ('exactly', 'less than',
    'greater than', 'between', 'not between');
  arda : ARRAY[0..6] OF String = ('1 day', '3 days', '1 week',
    '1 month', '3 months', '6 months', '1 year');
  arsu : ARRAY[0..2] OF String = ('bytes', 'KB', 'MB');
  arbt : ARRAY[0..2] OF String = ('moving them to the Recycle bin',
    'backing them up to a holding folder',
    'deleting them permanently');
VAR
  N : Integer;

  procedure reOut(vStyle : TFontStyles; vSize : Integer;
    vRets : Integer; const S : String);
  // Add some text to the RichEdit, possibly bold or underlined, with
  // specified indent and specified number of returns afterward.
  VAR N : Integer;
  begin
    WITH vRE.SelAttributes DO
      begin
        Style := vStyle;
        Size  := vSize;
      end;
    vRE.SelText := S;
    FOR N := 1 TO vRets DO
      vRE.SelText := #13#10;
  end;


begin
  vRE.ReadOnly := True;
  vRE.Clear;
  reOut(BB, 14, 0, 'Type: ');
  reOut(BU, 14, 0, fName);
  IF fChecked THEN
    reOut(BB, 14, 2, ' (enabled)')
  ELSE reOut(BB, 14, 2, ' (disabled)');

  IF (fDirs.Count=0) OR ((fDirs.Count=1) AND
    (Pos('Local drives (', fDirs[0]) = 1)) THEN
    reOut(II, 12, 1, 'Search '+NameForAll)
  ELSE
    begin
      IF fDirs.Count = 1 THEN
        reOut(II, 12, 1, 'Search for files in this folder:')
      ELSE reOut(II, 12, 1, 'Search for files in these folders:');
      FOR N := 0 TO fDirs.Count-1 DO
        begin
          reOut(xx, 10, 0, #9+fDirs[N]);
          CASE Integer(fDirs.Objects[N]) OF
            1 : reOut(II,10,1,#9' but not its subfolders');
            2 : reOut(II,10,1,#9'''s subfolders only');
            ELSE reOut(II,10,1,#9' and its subfolders');
          END;
        end;
    end;
  IF fDeletes.Count = 1 THEN
    reOut(II, 12, 1, 'Search for files matching this file '+
      'specification:')
  ELSE reOut(II, 12, 1, 'Search for files matching these file '+
    'specifications:');
  FOR N := 0 TO fDeletes.Count-1 DO
    reOut(xx, 10, 1, #9+fDeletes[N]);
  IF fDateRestrict OR fAgeRestrict OR fSizeRestrict THEN
    reOut(II, 12, 1, 'Restrict file matching by date or size:');
  IF fDateRestrict THEN
    begin
      reOut(xx, 10, 0, #9'File date must be ');
      IF fDateRestrictType <= 2 THEN
        begin
          reOut(xx, 10, 0, ard1[fDateRestrictType]+' ');
          reOut(BB, 10, 1, FormatDateTime('m/d/yyyy', fDateData1));
        end
      ELSE
        begin
          reOut(xx, 10, 0, ard1[fDateRestrictType]+' ');
          reOut(BB, 10, 0, FormatDateTime('m/d/yyyy', fDateData1));
          reOut(xx, 10, 0, ' and ');
          reOut(BB, 10, 1, FormatDateTime('m/d/yyyy', fDateData2));
        end;
    end;
  IF fAgeRestrict THEN
    begin
      reOut(xx, 10, 0, #9'File must be ');
      IF fAgeRestrictType <= 2 THEN
        begin
          reOut(xx, 10, 0, ard2[fAgeRestrictType]+' ');
          reOut(BB, 10, 0, arda[fAgeData1]);
        end
      ELSE
        begin
          reOut(xx, 10, 0, ard2[fAgeRestrictType]+' ');
          reOut(BB, 10, 0, arda[fAgeData1]);
          reOut(xx, 10, 0, ' and ');
          reOut(BB, 10, 0, arda[fAgeData2]);
        end;
      reOut(xx, 10, 1, ' old');
    end;
  IF fSizeRestrict THEN
    begin
      reOut(xx, 10, 0, #9'File size must be ');
      IF fSizeRestrictType <= 2 THEN
        begin
          reOut(xx, 10, 0, ard3[fSizeRestrictType]+' ');
          reOut(BB, 10, 0, IntToStr(fSizeData1)+' ');
        end
      ELSE
        begin
          reOut(xx, 10, 0, ard3[fSizeRestrictType]+' ');
          reOut(BB, 10, 0, IntToStr(fSizeData1)+' ');
          reOut(xx, 10, 0, 'and ');
          reOut(BB, 10, 0, IntToStr(fSizeData2)+' ');
        end;
      reOut(xx, 10, 1, arsu[fSizeUnit]);
    end;
  IF FProts.Count > 0 THEN
    begin
      IF fProts.Count = 1 THEN
        reOut(II, 12, 1, 'Protect files matching this file '+
          'specification:')
      ELSE reOut(II, 12, 1, 'Protect files matching these file '+
        'specifications:');
      FOR N := 0 TO fProts.Count-1 DO
        reOut(xx, 10, 1, #9+fProts[N]);
    end;
  reOut(xx, 10, 1, '');
  reOut(II, 12, 1, 'Clean up matching files by:');
  reOut(xx, 10, 1, #9+arbt[fBackType]);
  vRE.SelStart := 0;
end;

procedure TJunkFileObj.SetBackType(const Value: Integer);
begin
  CASE Value OF
    0..2 : fBackType := Value;
    ELSE Raise(Exception.Create('Invalid backup type'));
  END;
end;

procedure TJunkFileObj.SetDateRestrictType(const Value: Integer);
begin
  CASE Value OF
    0..4 : fDateRestrictType := Value;
    ELSE Raise(Exception.Create('Invalid date restriction'));
  END;
end;

procedure TJunkFileObj.SetDeletes(const Value: TStringList);
begin
  fDeletes.Assign(Value);
end;

procedure TJunkFileObj.SetDirs(const Value: TStringList);
begin
  fDirs.Assign(Value);
end;

procedure TJunkFileObj.SetProts(const Value: TStringList);
begin
  fProts.Assign(Value);
end;

procedure TJunkFileObj.SetSizeRestrictType(const Value: Integer);
begin
  CASE Value OF
    0..4 : fSizeRestrictType := Value;
    ELSE Raise(Exception.Create('Invalid size restriction'));
  END;
end;

procedure TJunkFileObj.SetSizeUnit(const Value: Integer);
begin
  CASE Value OF
    0..2 : fSizeUnit := Value;
    ELSE Raise(Exception.Create('Invalid Size unit'));
  END;
end;

function TJunkFileObj.SameAs(TJ: TJunkFileObj): Boolean;
VAR N : Integer;
begin
  Result     := False;
  // We ignore fChecked
  // We ignore fName
  IF fBacktype          <> TJ.fBacktype THEN Exit;
  IF fDateRestrict      <> TJ.fDateRestrict      THEN Exit;
  IF fDateRestrictType  <> TJ.fDateRestrictType  THEN Exit;
  IF fDateData1         <> TJ.fDateData1         THEN Exit;
  IF fDateData2         <> TJ.fDateData2         THEN Exit;
  IF fAgeRestrict       <> TJ.fAgeRestrict       THEN Exit;
  IF fAgeRestrictType   <> TJ.fAgeRestrictType   THEN Exit;
  IF fAgeData1          <> TJ.fAgeData1          THEN Exit;
  IF fAgeData2          <> TJ.fAgeData2          THEN Exit;
  IF fSizeRestrict      <> TJ.fSizeRestrict      THEN Exit;
  IF fSizeRestrictType  <> TJ.fSizeRestrictType  THEN Exit;
  IF fSizeData1         <> TJ.fSizeData1         THEN Exit;
  IF fSizeData2         <> TJ.fSizeData2         THEN Exit;
  IF fSizeUnit          <> TJ.fSizeUnit          THEN Exit;
  IF fProts.Count   <> TJ.fProts.Count THEN Exit;
  IF fDirs.Count    <> TJ.fDirs.Count THEN Exit;
  IF fDeletes.Count <> TJ.fDeletes.Count THEN Exit;
  FOR N := 0 TO fProts.Count-1 DO
    IF AnsiCompareText(fProts[N], TJ.fProts[N]) <> 0 THEN
      Exit;
  FOR N := 0 TO fDeletes.Count-1 DO
    IF AnsiCompareText(fDeletes[N], TJ.fDeletes[N]) <> 0 THEN
      Exit;
  FOR N := 0 TO fDirs.Count-1 DO
    IF (AnsiCompareText(fDirs[N], TJ.fDirs[N]) <> 0) OR
      (fDirs.Objects[N] <> TJ.fDirs.Objects[N]) THEN
      Exit;
  Result := True;
end;

procedure TJunkFileObj.SetAgeRestrictType(const Value: Integer);
begin
  CASE Value OF
    0..4 : fSizeRestrictType := Value;
    ELSE Raise(Exception.Create('Invalid age restriction'));
  END;
end;

constructor TJunkFileObj.Create(const vName: String; vChecked:
  Boolean; vBacktype: Integer; vDirs, vDeletes, vProts: String;
  Onlies : DWORD);
VAR
  N : Integer;
begin
  fName      := vName;
  fChecked   := vChecked;
  fBacktype  := vBacktype;
  fDirs      := TStringList.Create;
  fDirs.Text := vDirs;
  FOR N := 0 TO fDirs.Count-1 DO
    begin
      IF Odd(Onlies) THEN
        fDirs.Objects[N] := Pointer(1);
      Onlies := Onlies DIV 2;
    end;
  fDeletes   := TStringList.Create;
  fDeletes.Text := vDeletes;
  fProts     := TStringList.Create;
  fProts.Text := vProts;
  fDateRestrict := FALSE;
  fDateRestrictType := 0;
  fDateData1 := 0.0;
  fDateData2 := 0.0;
  fAgeRestrict := False;
  fAgeRestrictType := 0;
  fAgeData1 := 0;
  fAgeData2 := 0;
  fSizeRestrict := False;
  fSizeRestrictType := 0;
  fSizeData1 := 0;
  fSizeData2 := 0;
  fSizeUnit := 0;
end;

procedure TJunkFileObj.ActOnFiles(DoWhat: TJDoFilType; DoReport:
  TJReptType; WhenDone : TNotifyEvent; vStopEvent : THandle;
  const vMyDir : String);
begin
  IF NOT Assigned(WhenDone) THEN Exit;
  WITH TJunkDelThread.Create(Self, DoWhat, DoReport, WhenDone,
    vStopEvent, vMyDir) DO Resume;
end;

function TJunkFileobj.MatchProts(const sD, sN : String) : Boolean;
VAR
  Ns   : String;
  N    : Integer;
  buff : ARRAY[0..MAX_PATH] OF Char;
begin
  StrPCopy(buff, sD+'\'+sN);
  GetShortPathname(buff, buff, MAX_PATH);
  Ns := ExtractFilename(StrPas(buff));
  Result := True;
  FOR N := 0 TO fProts.Count-1 DO
    begin
      IF Pos(':', fProts[N]) = 2 THEN
        begin
          IF (AnsiCompareText(fProts[N], sD + '\' + sN) = 0) OR
             (AnsiCompareText(fProts[N], sD + '\' + Ns) = 0) THEN
            Exit;
        end
      ELSE
        begin
          IF MyPathMatchSpec(sN, fProts[N]) OR
            MyPathMatchSpec(Ns, fProts[N]) THEN Exit;
        end;
    end;
  Result := False;
end;

function TJunkFileObj.MatchRestrict(TD: TWin32FindData): Boolean;
// Returnst true if the file matches all file restrictions
const
  UnitMul : ARRAY[0..2] OF Int64 = (1, 1024, 1024*1024);
VAR
  timF : TDateTime;

  procedure SubYear(VAR T : TDateTime; vY : Integer);
  VAR M, D, Y : Word;
  begin
    DecodeDate(T, Y, M, D);
    T := EncodeDate(Y-vY, M, D);
  end;

  procedure SubMonth(VAR T : TDateTime; vM : Integer);
  VAR M, D, Y : Word;
  begin
    DecodeDate(T, Y, M, D);
    IF M > vM THEN
      T := EncodeDate(Y, M-vM, D)
    ELSE
      T := EncodeDate(Y-1, M+12-vM, D);
  end;

  procedure SubDay(VAR T : TDateTime; vD : Integer);
  begin
    T := T - vD;
  end;

  function GotDate : Boolean;
  VAR
    FLoc : TFileTime;
    STmp : TSystemTime;
  begin
    Result := False;
    IF NOT FileTimeToLocalFileTime(TD.ftLastWriteTime, FLoc) THEN
      Exit;
    IF NOT FileTimeToSystemTime(FLoc, STmp) THEN Exit;
    try
      TimF := SystemTimeToDateTime(STmp);
      TimF := Int(TimF);
    except
      ON EConvertError DO Exit;
    end;
    Result := True;
  end;

  function DateNoMatch : Boolean;
  begin
    Result := False;
    CASE fDateRestrictType OF
      0 : IF Abs(TimF-fDateData1) < 0.01 THEN Exit; // exactly
      1 : IF TimF < fDateData1 THEN Exit; // before
      2 : IF TimF >= fDateData1 THEN Exit; // after
      3 : IF (TimF >= fDateData1) AND
             (TimF < fDateData2) THEN Exit; // between
      4 : IF (TimF < fDateData1) OR
             (TimF >= fDateData2) THEN Exit; // not between
    END;
    Result := True;
  end;

  function AgeNoMatch : Boolean;
  VAR tim1, tim2 : TDateTime;
  begin
    Result := False;
    tim1 := Date;
    tim2 := Date;
    CASE fAgeData1 OF
      0 : SubDay(tim1, 1);
      1 : SubDay(tim1, 3);
      2 : SubDay(tim1, 7);
      3 : SubMonth(tim1, 1);
      4 : SubMonth(tim1, 3);
      5 : SubMonth(tim1, 6);
      6 : SubYear(tim1, 1);
    END;
    CASE fAgeData2 OF
      0 : SubDay(tim2, 1);
      1 : SubDay(tim2, 3);
      2 : SubDay(tim2, 7);
      3 : SubMonth(tim2, 1);
      4 : SubMonth(tim2, 3);
      5 : SubMonth(tim2, 6);
      6 : SubYear(tim2, 1);
    END;
    CASE fAgeRestrictType OF
      0 : IF timF = tim1 THEN Exit; // exactly
      1 : IF timF < tim1 THEN Exit; // older
      2 : IF timF >= tim1 THEN Exit; // newer
      3 : IF (timF >= tim2) AND
             (timF < tim1) THEN Exit; // between
      4 : IF (timF < tim2) OR
             (timF >= tim1) THEN Exit; // not between
    END;
    Result := True;
  end;

  function SizeNoMatch : Boolean;
  VAR sizF, siz1, siz2 : Int64;
  begin
    Result := False;
    sizF := TD.nFileSizeHigh;
    sizF := (sizF SHL 32) + TD.nFileSizeLow;
    siz1 := fSizeData1 * UnitMul[fSizeUnit];
    siz2 := fSizeData2 * UnitMul[fSizeUnit];
    CASE fSizeRestrictType OF
      0 : IF sizF = siz1 THEN Exit; // exactly
      1 : IF sizF < siz1 THEN Exit; // less
      2 : IF sizF >= siz1 THEN Exit; // greater
      3 : IF (sizF >= siz1) AND
             (sizF < siz2) THEN Exit; // between
      4 : IF (sizF < siz1) OR
             (sizF >= siz2) THEN Exit; // not between
    END;
    Result := True;
  end;

begin
  IF fDateRestrict AND GotDate AND DateNoMatch THEN
    Result := False
  ELSE IF fAgeRestrict AND GotDate AND AgeNoMatch THEN
    Result := False
  ELSE IF fSizeRestrict AND SizeNoMatch THEN
    Result := False
  ELSE Result := True;
end;

constructor TJunkFileObj.Create(TJ: TJunkFileObj);
begin
  fName             := TJ.fName;
  fChecked          := TJ.fChecked;
  fBacktype         := TJ.fBacktype;
  fDirs             := TStringList.Create;
  fDeletes          := TStringList.Create;
  fProts            := TStringList.Create;
  fDirs.Assign(   TJ.fDirs);
  fDeletes.Assign(TJ.fDeletes);
  fProts.Assign(  TJ.Prots);
  fDateRestrict     := TJ.fDateRestrict;
  fDateRestrictType := TJ.fDateRestrictType;
  fDateData1        := TJ.fDateData1;
  fDateData2        := TJ.fDateData2;
  fAgeRestrict      := TJ.fAgeRestrict;
  fAgeRestrictType  := TJ.fAgeRestrictType;
  fAgeData1         := TJ.fAgeData1;
  fAgeData2         := TJ.fAgeData2;
  fSizeRestrict     := TJ.fSizeRestrict;
  fSizeRestrictType := TJ.fSizeRestrictType;
  fSizeData1        := TJ.fSizeData1;
  fSizeData2        := TJ.fSizeData2;
  fSizeUnit         := TJ.fSizeUnit;
end;

{ TJunkDelThread }

constructor TJunkDelThread.Create(vTJ : TJunkFileObj; vDoWhat :
  TJDoFilType; vReport : TJReptType; vWhenDone : TNotifyEvent;
  vStopEvent : THandle; const vMyDir : String);
begin
  Inherited Create(True);
  FreeOnTerminate := True;
  fTJ             := vTJ;
  fDoWhat         := vDoWhat;
  fReport         := vReport;
  OnTerminate     := vWhenDone;
  fStopEvent      := vStopEvent;
  fStopped        := False;
  fMyDir          := vMyDir;
  Priority        := tpLower;
end;

procedure TJunkDelThread.Execute;
// In processing files, it's most efficient to do FindFirst/Next on
// each "deletes" filespec, then use pattern matching to remove the
// unwanted.
VAR
  TempS : TStringList;
  N     : Integer;
  S     : String;

  function HasNoExt(const S : String) : Boolean;
  VAR Ext : String;
  begin
    Ext := ExtractFileExt(S);
    Result := (Ext = '.') OR (Ext = '');
  end;

  procedure ProcessFiles(const sDir : String);
  VAR
    FD    : TWin32FindData;
    FH    : THandle;
    N     : Integer;
    NoExt : Boolean;
    Found : TStringList;
  begin
    Found := TStringList.Create;
    Found.Sorted := True;
    try
      FOR N := 0 TO fTJ.fDeletes.Count-1 DO
        begin
          NoExt := HasNoExt(ftJ.fDeletes[N]);
          FH := FindFirstFile(PChar(sDir + '\' + fTJ.fDeletes[N]),
            FD);
          IF FH = INVALID_HANDLE_VALUE THEN Continue;
          try
            REPEAT
              IF WaitForSingleObject(fStopEvent, 0) =
                WAIT_OBJECT_0 THEN
                begin
                  fStopped := True;
                  Break;
                end;
              IF FD.dwFileAttributes AND FILE_ATTRIBUTE_DIRECTORY > 0
                THEN Continue;
              IF NoExt AND (NOT HasNoExt(FD.cFilename))
                THEN Continue;
              IF Found.IndexOf(FD.cFilename) = -1 THEN
                Found.Add(FD.cFilename)
              ELSE Continue;
              Inc(fNumDel);
              IF NOT fTJ.MatchRestrict(FD) THEN
                begin
                  Inc(fNumRes);
                  Continue;
                end;
              IF fTJ.MatchProts(sDir, FD.cFilename) THEN
                begin
                  Inc(fNumProt);
                  Continue;
                end;
              fTmpSiz := FD.nFileSizeHigh;
              fTmpSiz := (fTmpSiz SHL 32) + FD.nFileSizeLow;
              fSizDel := fSizDel + fTmpSiz;
              IF Assigned(fDoWhat) THEN
                begin
                  fTmp1 := sDir + '\' + FD.cFilename;
                  SyncDoWhat;
                end;
            UNTIL (NOT FindNextFile(FH, FD)) OR fStopped;
          finally
            Windows.FindClose(FH);
          end;
        end;
    finally
      Found.Free;
    end;
  end;

  procedure ProcessDir(const sDir : String; How : Integer);
  VAR
    FD : TWin32FindData;
    FH : THandle;
  begin
    IF AnsiCompareText(sDir, fMyDir) = 0 THEN
      Exit;
    IF How <> 2 THEN
      begin
        ProcessFiles(sDir);
        Inc(fNumDir);
      end;
    IF How = 1 THEN Exit;
    FH := FindFirstFile(PChar(sDir+'\*.*'), FD);
    IF FH = INVALID_HANDLE_VALUE THEN Exit;
    try
      REPEAT
        IF WaitForSingleObject(fStopEvent, 0) = WAIT_OBJECT_0 THEN
          begin
            fStopped := True;
            Break;
          end;
        IF FD.dwfileAttributes AND FILE_ATTRIBUTE_DIRECTORY = 0 THEN
          Continue;
        IF StrComp('.', FD.cFilename) = 0 THEN Continue;
        IF StrComp('..', FD.cFilename) = 0 THEN Continue;
        IF (Length(sDir) = 2) AND
           (StrIComp(FD.cFilename, 'HDVAL$$$.$$$') = 0) THEN
           Continue;
        IF (Length(sDir) = 2) AND
           ((StrIComp(FD.cFilename, 'RECYCLED') = 0) OR
            (StrIComp(FD.cFilename, 'RECYCLER') = 0)) AND
           (FD.dwfileAttributes AND FILE_ATTRIBUTE_SYSTEM > 0) AND
           (FD.dwfileAttributes AND FILE_ATTRIBUTE_HIDDEN > 0) THEN
          Continue;
        ProcessDir(sDir+'\'+StrPas(FD.cFilename), 0);
      UNTIL (NOT FindNextFile(FH, FD)) OR fStopped;
    finally
      Windows.FindClose(FH);
    end;
  end;

begin
  fNumDel  := 0;
  fNumRes  := 0;
  fNumProt := 0;
  fNumDir  := 0;
  fSizDel  := 0;
  TempS := TStringList.Create;
  try
    IF (fTJ.fDirs.Count = 0) OR ((fTJ.fDirs.Count = 1) AND
      (Pos('Local drives (', fTJ.fDirs[0]) = 1)) THEN
      begin
        TempS.Text := LocalDrives(':'#13#10,'', IgnoreSubst); //1.01
      end
    ELSE
      begin
        TempS.Assign(fTJ.fDirs);
        FOR N := 0 TO TempS.Count -1 DO
          begin
            S := TempS[N];
            IF S[length(S)] = '\' THEN
              begin
                SetLength(S, Length(S)-1);
                tempS[N] := S;
              end;
          end;
      end;
    FOR N := 0 TO TempS.Count-1 DO
      begin
        IF WaitForSingleObject(fStopEvent, 0) = WAIT_OBJECT_0 THEN
          begin
            fStopped := True;
            Break;
          end;
        ProcessDir(TempS[N], Integer(TempS.Objects[N]));
      end;
    IF (NOT fStopped) AND Assigned(freport) THEN
      Synchronize(SyncReport);
  finally
    TempS.Free;
  end;
end;

procedure TJunkDelThread.SyncReport;
begin
  fReport(fSizDel, fNumDir, fNumDel, fNumRes, fNumProt);
end;

procedure TJunkDelThread.SyncDoWhat;
begin
  fDoWhat(fTmp1, fTmpSiz);
end;

procedure AddAllDefaults(LV : TListView; RecycleProb : Boolean);
VAR
  N : Integer;
  S : String;
  Buffer : ARRAY[0..MAX_PATH] OF Char;
  Meth : Integer;
begin
  IF RecycleProb THEN Meth := cuHolding
  ELSE Meth := cuRecycle;
  WITH LV.Items DO
    begin
      Add.Data := TJunkFileObj.Create('Backup files',
        True, Meth, '', '*.bak'#13#10'*.old', '', 0);
      Add.Data := TJunkFileObj.Create('Borland C++ temporary files',
        True, Meth, '', '*.ilc'#13#10'*.ild'#13#10'*.ilf'+
        #13#10'*.ils'#13#10'*.tds', '', 0);
      Add.Data := TJunkFileObj.Create('Help temporary files',
        True, Meth, '', '*.gid'#13#10'*.fts'#13#10'*.ftg', '',
        0);
      S := IECache;
      IF S <> '' THEN
        Add.Data := TJunkFileObj.Create('Internet Explorer cache',
          True, cuDelete, S, '*.*', 'index.dat', 0);
      Add.Data := TJunkFileObj.Create('Lost cluster files',
        True, Meth, '', 'chklist.*'#13#10'*.chk'#13#10+
        'file???._dd'#13#10'file????._dd', 'chklist.dll', 0);
      Add.Data := TJunkFileObj.Create('Setup temporary files',
        True, Meth, '', 'mscreate.dir'#13#10'*.---', '', 0);
      GetTempPath(MAX_PATH, Buffer);
      S := StrPas(Buffer);
      IF Length(S) < 4 THEN S := ''; //1.01
      IF S <> '' THEN
        Add.Data := TJunkFileObj.Create('Temp folders',
          True, Meth, S, '*.*', '', 1);
      Add.Data := TJunkFileObj.Create('Temporary files',
        True, Meth, '', '*.tmp'#13#10'*.~*'#13#10'~*.*'+
        #13#10'*.^*'#13#10'*.??$', 'rcveak*.cp$'#13#10'~desktop.*',
        0);
      Add.Data := TJunkFileObj.Create('Visual Studio temp files',
        True, Meth, '', '*.bsc'#13#10'*.ilk'#13#10'*.res'+
        #13#10'*.pch', '', 0);
      GetWindowsDirectory(Buffer, MAX_PATH);
      Buffer[3] := #0;
      S := StrPas(Buffer);
      IF S <> '' THEN
        Add.Data := TJunkFileObj.Create('Windows diagnostic files',
          True, Meth, S, 'bootlog.prv'#13#10'modemdet.txt'+
          #13#10'suhdlg.dat'#13#10'system.1st'#13#10'*log.txt', '', 0);
      Add.Data := TJunkFileObj.Create('WS_FTP temporary files',
        True, Meth, '', 'ws_ftp.log', '', 0);
    end;
  FOR N := 0 TO LV.Items.Count-1 DO
    WITH LV.Items[N] DO
      WITH TJunkFileObj(Data) DO
        begin
          Caption := typName;
          Checked := IsChecked;
        end;
end;

end.
