unit nufpicku;

// Copyright (c) 1999 by Ziff-Davis, Inc.
// Written by Neil J. Rubenking

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ImgList, StdCtrls, ExtCtrls, ComCtrls, ActnList,
  legendu, Buttons;

type
  TNewPickForm = class(TForm)
    pnlFullPath    : TPanel;
    tvDirSel       : TTreeView;
    imDirSel       : TImageList;
    ActionList1    : TActionList;
      aFolder      : TAction;
      aAllSubs     : TAction;
      aExistSubs   : TAction;
      aNoSubs      : TAction;
      aLegend      : TAction;
    pmDirSel       : TPopupMenu;
      mnuFolder    : TMenuItem;
      N1           : TMenuItem;
      mnuInclude   : TMenuItem;
      mnuExclude   : TMenuItem;
      mnuListed    : TMenuItem;
      N2           : TMenuItem;
      mnuLegend    : TMenuItem;
    pnlButtons     : TPanel;
      btnInclude   : TBitBtn;
      btnExclude   : TBitBtn;
      btnListed    : TBitBtn;
      btnOK        : TButton;
      btnCancel    : TButton;
      btnLegend    : TButton;
    procedure FormCreate(Sender: TObject);
    procedure tvDirSelChange(Sender: TObject; Node: TTreeNode);
    procedure tvDirSelCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure tvDirSelExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure tvDirSelGetSelectedIndex(Sender: TObject;
      Node: TTreeNode);
    procedure tvDirSelKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure tvDirSelKeyPress(Sender: TObject; var Key: Char);
    procedure tvDirSelMouseDown(Sender: TObject; Button:
      TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure tvDirSelMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure aAllSubsUpdate(Sender: TObject);
    procedure aFolderExecute(Sender: TObject);
    procedure aAllSubsExecute(Sender: TObject);
    procedure aNoSubsExecute(Sender: TObject);
    procedure aExistSubsExecute(Sender: TObject);
    procedure aLegendExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    NameForAll : String;
    OverCkBox  : Boolean;
    MyLegend   : TLegendForm;
    function AnySubdirs(const S : String) : Boolean;
    procedure SetOneSub(vTN : TTreeNode; valu : Integer);
    procedure SetOneFol(vTN : TTreeNode; cked : Boolean);
    function  FullPath(Node : TTreeNode) : String;
    function  TreeRoot : TTreeNode;
    function  GetSub(vTN : TTreeNode) : Integer;
    procedure SetFolAndAdjust(vTN : TTreeNode; idx : Integer);
    procedure SetSubAndAdjust(vTN : TTreeNode; idx : Integer);
    function  FolChecked(vTN : TTreeNode) : Boolean;
    procedure ParSet(vTN : TTreeNode; vIsCk, vNoCk : Boolean);
    function  AllSibsMatch(vpar : TTreeNode;
      IsChecked : Boolean) : Boolean;
  public
    { Public declarations }
    procedure SaveToList(TS : TStrings);
    procedure LoadFromList(TS : TStrings);
  end;

var
  NewPickForm: TNewPickForm;

implementation

uses allfuncs, hdobjs {1.01};

{$R *.DFM}
const
  cSubChkP = 0; // ALL subfolders
  cSubNotP = 1; // NO subfolders
  cSubChek = 2; // Listed subfolders
  cSubSome = 3; // Some (not all) subfolders
  cSubGray = 4; // no subfolders exist
  cSubLO   = cSubChkP;
  cSubHI   = cSubGray;
  cFolderY = 5; // Folder checked
  cFolderN = 6; // Folder unchecked
  pFolOnly = Pointer(1);
  pSubOnly = Pointer(2);

procedure TNewPickForm.FormCreate(Sender: TObject);
VAR
  S          : String;
  tvRoot, TN : TTreeNode;
  N          : Integer;
begin
  GetPosFmINI(ChangeFileExt(Application.ExeName, '.INI'),
    Self, True);
  MyLegend := TLegendForm.Create(Self);
  MyLegend.Visible := False;
  tvRoot      := tvDirSel.Items.Add(nil, 'Local Drives');
  NameForAll  := LocalDrives(':', ',', IgnoreSubst); //1.01
  S           := LocalDrives('','', IgnoreSubst); //1.01
  FOR N := 1 TO Length(S) DO
    begin
      TN := tvDirSel.Items.AddChild(tvRoot, S[N]+':');
      TN.HasChildren := AnySubdirs(TN.Text);
      SetOneSub(TN, cSubChkP);
      SetOneFol(TN, True);
    end;
  IF NameForAll <> '' THEN
    SetLength(NameForAll, Length(NameForAll)-1);
  tvRoot.Expand(False);
  SetOneSub(tvRoot, cSubChkP);
end;

procedure TNewPickForm.FormDestroy(Sender: TObject);
begin
  SetPosToINI(ChangeFileExt(Application.ExeName, '.INI'),
    Self, True);
end;

procedure TNewPickForm.tvDirSelChange(Sender: TObject;
  Node: TTreeNode);
begin // Display selected node in panel
  IF Node = nil THEN
    pnlFullPath.Caption := ''
  ELSE
    pnlFullPath.Caption := ' ' + FullPath(Node);
end;

procedure TNewPickForm.tvDirSelCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin // don't collapse the root
  AllowCollapse := Node <> TreeRoot;
end;

procedure TNewPickForm.tvDirSelExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
VAR
  S    : String;
  TS   : TSearchRec;
  Rslt : Integer;
  TN   : TTreeNode;
begin
// If this node has never been expanded, create its children now
  IF Node.HasChildren AND (Node.Count = 0) THEN
    begin
      S := FullPath(Node);
      Rslt := FindFirst(S+'*.*', faAnyFile, TS);
      WHILE Rslt = 0 DO
        begin
          IF (TS.Attr AND faDirectory <> 0) AND (TS.Name <> '') AND
             (TS.Name[1] <> '.') THEN
            begin
              TN := tvDirSel.Items.AddChild(Node, TS.Name);
              IF AnySubdirs(S+TN.Text) THEN
                begin
                  TN.HasChildren := True;
                  // The parent node will always be cSubChkP or
                  // cSubNotP. Set the child the same.
                  SetOneSub(TN, GetSub(Node));
                end
              ELSE
                begin
                  TN.HasChildren := False;
                  SetOneSub(TN, cSubGray);
                end;
              IF GetSub(Node) = cSubChkP THEN
                SetOneFol(TN, True)
              ELSE SetOneFol(TN, False);
            end;
          Rslt := FindNext(TS);
        end;
      FindClose(TS);
      IF Node.Count = 0 THEN Node.HasChildren := False
      ELSE Node.AlphaSort;
    end;
end;

procedure TNewPickForm.tvDirSelGetSelectedIndex(Sender: TObject;
  Node: TTreeNode);
begin  // use the same icon whether selected or not
  Node.SelectedIndex := Node.ImageIndex;
end;

procedure TNewPickForm.tvDirSelKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin // Suppress Alt+* "expand all" keystroke
  IF key = $6A THEN Key := 0;
end;

procedure TNewPickForm.tvDirSelKeyPress(Sender: TObject;
  var Key: Char);
begin // When spacebar is pressed, toggle folder checkbox
  IF key <> ' ' THEN Exit;
  IF tvDirSel.Selected = nil THEN Exit;
  SetFolAndAdjust(tvDirSel.Selected, -1);
end;

procedure TNewPickForm.tvDirSelMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
// The expand/collapse happens before this event, so we track
// whether the cursor is over the checkbox in OnMouseMove
begin
  IF button = mbRight THEN // Select on right-click
    tvDirSel.Selected := tvDirSel.GetNodeAt(X,Y);
  IF button <> mbleft        THEN Exit;
  IF tvDirSel.Selected = nil THEN Exit;
  IF OverCkBox THEN SetFolAndAdjust(tvDirSel.Selected, -1);
end;

procedure TNewPickForm.tvDirSelMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  OverCkBox := htOnStateIcon IN tvDirSel.GetHitTestInfoAt(X, Y);
end;

procedure TNewPickForm.aAllSubsUpdate(Sender: TObject);
begin
  aLegend.Checked := MyLegend.Visible;
  IF tvDirSel.Selected = nil THEN
    begin
      aFolder.Enabled    := False;
      aAllSubs.Enabled   := False;
      aNoSubs.Enabled    := False;
      aExistSubs.Enabled := False;
    end
  ELSE
    begin
      IF tvDirSel.Selected = TreeRoot THEN
        begin
          aFolder.Enabled    := False;
          aFolder.Checked    := False;
          aAllSubs.Enabled   := True;
          aNoSubs.Enabled    := True;
          aExistSubs.Enabled := True;
        end
      ELSE
        begin
          aFolder.Enabled    := True;
          aFolder.Checked    := FolChecked(tvDirSel.Selected);
          // The ALL action is enabled for any folder that has
          // childen, or for childless folders whose parent
          // is not set to ALL
          aAllSubs.Enabled   := tvDirSel.Selected.HasChildren OR
            (GetSub(tvDirSel.Selected.Parent) <> cSubChkP);
          // The NO action is enabled for any folder that has
          // children
          aNoSubs.Enabled    := tvDirSel.Selected.HasChildren;
          aExistSubs.Enabled := True;
        end;
    end;
end;

procedure TNewPickForm.aFolderExecute(Sender: TObject);
begin
  IF tvDirSel.Selected = nil THEN Exit;
  SetFolAndAdjust(tvDirSel.Selected, -1);
end;

procedure TNewPickForm.aAllSubsExecute(Sender: TObject);
begin
  IF tvDirSel.Selected = nil THEN Exit;
  SetSubAndAdjust(tvDirSel.Selected, cSubChkP);
end;

procedure TNewPickForm.aNoSubsExecute(Sender: TObject);
begin
  IF tvDirSel.Selected = nil THEN Exit;
  SetSubAndAdjust(tvDirSel.Selected, cSubNotP);
end;

procedure TNewPickForm.aExistSubsExecute(Sender: TObject);
VAR Par : TTreeNode;
begin
  IF tvDirSel.Selected = nil THEN Exit;
  // Before we can select listed subroutines, we have to make
  // sure that they've been loaded into the tree
  tvDirSel.Selected.Expand(False);
  IF tvDirSel.Selected.HasChildren THEN
    begin
      SetSubAndAdjust(tvDirSel.Selected, cSubChek);
      // Walk the tree up toward the root as long as
      // the parent nodes are set to ALL; change each
      // to "Listed"
      par := tvDirSel.Selected.Parent;
      WHILE par <> nil DO
        begin
          IF GetSub(Par) = cSubChkP THEN
            SetOneSub(Par, cSubChek);
          par := par.parent;
        end;
    end
  ELSE SetSubAndAdjust(tvDirSel.Selected, cSubGray);
  tvDirSel.Refresh;
  btnOK.Enabled := (GetSub(TreeRoot) <> cSubNotP);
end;

procedure TNewPickForm.aLegendExecute(Sender: TObject);
// Toggle the displayed state of the legend. If it is becoming
// visible, set it to alwaysy-on-top and line it up with this
// window's top right corner
begin
  IF NOT MyLegend.Visible THEN
    SetWindowPos(MyLegend.Handle, HWND_TOPMOST, 0, 0, 0, 0,
      SWP_NOSIZE OR SWP_NOMOVE);
  MyLegend.Visible := NOT MyLegend.Visible;
end;

function TNewPickForm.AnySubdirs(const S : String) : Boolean;
// True if the passed folder has at least one subfolder
VAR
  TS   : TSearchRec;
  Rslt : Integer;
begin
  Result := False;
  Rslt := FindFirst(S+'\*.*', faAnyFile, TS);
  WHILE Rslt = 0 DO
    begin
      IF (TS.Attr AND faDirectory <> 0) AND (TS.Name <> '') AND
         (TS.Name[1] <> '.') THEN
        begin
          Result := True;
          Break;
        end;
      Rslt := FindNext(TS);
    end;
  FindClose(TS);
end;

procedure TNewPickForm.SetOneSub(vTN: TTreeNode; valu: Integer);
// Set one node's ImageIndex property to show the folder icon
begin
  IF (valu < cSubLO) OR (valu > cSubHI) THEN Exit;
  vTN.ImageIndex := valu;
end;

procedure TNewPickForm.SetOneFol(vTN: TTreeNode; cked: Boolean);
// Set one node's folder checkbox to checked or not
begin
  IF cked THEN vTN.StateIndex := cFolderY
  ELSE vTN.StateIndex := cFolderN;
end;

function TNewPickForm.FullPath(Node : TTreeNode) : String;
// Return the full path corresponding to a given node
VAR P : TTreeNode;
begin
  Result := '';
  P := Node;
  WHILE P.Parent <> nil DO
    begin
      Result := P.Text + '\' + Result;
      P := P.Parent;
    end;
end;

function TNewPickForm.TreeRoot: TTreeNode;
begin
  TreeRoot := tvDirSel.Items.GetFirstNode;
end;

function TNewPickForm.GetSub(vTN: TTreeNode): Integer;
begin
  Result := Integer(vTN.ImageIndex);
end;

procedure TNewPickForm.SetFolAndAdjust(vTN: TTreeNode; idx: Integer);
// Set folder status specifically to cFolderY or cFolderN.
// If idx is -1 toggle status
VAR IsCk : Boolean;
begin
  IF vTN = TreeRoot THEN Exit; // can't check/uncheck the root
  CASE idx OF
    cFolderY : IsCk := True;
    cFolderN : IsCk := False;
    -1     : IsCk := NOT FolChecked(vTN);
    ELSE Exit;
  END;
  SetOneFol(vTN, IsCk);
  // Propagate the change up the tree
  ParSet(vTN, IsCk, NOT IsCk);
  tvDirSel.Refresh;
  btnOK.Enabled := (GetSub(TreeRoot) <> cSubNotP);
end;

procedure TNewPickForm.SetSubAndAdjust(vTN: TTreeNode; idx: Integer);

  procedure ChdSet(vTN : TTreeNode; IsCk : Boolean);
  // Set all children of this node to specified index
  VAR chd : TTreeNode;
  begin
    chd := vTN.getFirstChild;
    WHILE chd <> nil DO
      begin
        IF chd.HasChildren THEN
          begin
            IF IsCk THEN
              SetOneSub(chd, cSubChkP)
            ELSE
              SetOneSub(chd, cSubNotP);
          end
        ELSE SetOneSub(chd, cSubGray);
        SetOneFol(chd, IsCk);
        ChdSet(Chd, IsCk);
        chd := vTN.GetNextChild(chd);
      end;
  end;

begin
  SetOneSub(vTN, Idx);
  // Propagate the "listed" state the same as ALL
  IF idx = cSubChek THEN idx := cSubChkP;
  // Propagate the "loner" state the same as ALL
  IF idx = cSubGray THEN idx := cSubChkP;
  // Propagate the change down the tree
  chdSet(vTN, Idx = cSubChkP);
  // Propagate the change up the tree
  ParSet(vTN, Idx = cSubChkP, idx = cSubNotP);
  tvDirSel.Refresh;
  btnOK.Enabled := (GetSub(TreeRoot) <> cSubNotP);
end;

function TNewPickForm.FolChecked(vTN: TTreeNode): Boolean;
begin
  Result := vTN.StateIndex = cFolderY;
end;

procedure TNewPickForm.ParSet(vTN : TTreeNode; vIsCk,
  vNoCk : Boolean);
// Change parent icon based on this node's icon. When the
// icon is "some", both vIsCk and vNoCk are FALSE
VAR par : TTreeNode;
begin
  par := vTN.Parent;
  IF par = nil THEN Exit;
  IF vIsCk AND vNoCk THEN Exit; // shouldn't happen
  CASE GetSub(par) OF
    cSubChek, cSubChkP : IF NOT vIsCk THEN
      begin
        IF AllSibsMatch(par, False) THEN
          SetOneSub(Par, cSubNotP)
        ELSE
          SetOneSub(par, cSubSome);
      end;
    cSubNotP : IF NOT vNoCk THEN
      begin
        IF AllSibsMatch(par, True) THEN
          SetOneSub(Par, cSubChek)
        ELSE
          SetOneSub(par, cSubSome);
      end;
    cSubSome : IF vNoCk AND AllSibsMatch(par, False) THEN
      SetOneSub(par, cSubNotP)
    ELSE IF vIsCk AND AllSibsMatch(par, True) THEN
      SetOneSub(par, cSubChek);
  END;
  ParSet(par, visCk, vNoCk);
end;

function TNewPickForm.AllSibsMatch(vpar : TTreeNode;
  IsChecked : Boolean) : Boolean;
// Checks whether all children of the specified node totally
// match the checked/unchecked state. That means the checkbox
// status must match IsChecked. The folder icon must be
// green-plus, green, or gray for IsChecked; red or gray for
// NOT IsChecked
VAR vsib : TTreeNode;
  function Matches(I : Integer) : Boolean;
  begin
    CASE I OF
      cSubChek,
      cSubChkP : Result := IsChecked;
      cSubNotP : Result := NOT IsChecked;
      cSubGray : Result := True;
      ELSE      Result := False;
    END;
  end;
begin
  vsib := vpar.GetFirstChild;
  result := True;
  WHILE vsib <> nil DO
    begin
      IF (NOT Matches(GetSub(vsib))) OR
         (FolChecked(vsib) <> IsChecked) THEN
        begin
          Result := False;
          Break;
        end;
      vsib := vpar.GetNextChild(vsib);
    end;
end;

procedure TNewPickForm.SaveToList(TS: TStrings);
// Save the smallest list possible that defines the selected
// folders.

  procedure Process(TN : TTreeNode; isRoot : Boolean);
  VAR chd : TTreeNode;
  begin
    CASE GetSub(TN) OF
      cSubChkP : begin
        IF isRoot THEN
          TS.Add(Format('Local drives (%s)', [NameForAll]))
        ELSE IF FolChecked(TN) THEN
          TS.Add(FullPath(TN))
        ELSE TS.AddObject(FullPath(TN), pSubOnly);
      end;
      cSubNotP : begin
        IF FolChecked(TN) THEN
          TS.AddObject(FullPath(TN), pFolOnly);
      end;
      cSubGray : begin
        IF FolChecked(TN) THEN
          TS.AddObject(FullPath(TN), pFolOnly);
      end;
      cSubChek,
      cSubSome : begin
        IF FolChecked(TN) THEN
          TS.AddObject(FullPath(TN), pFolOnly);
        chd := TN.GetFirstChild;
        WHILE chd <> nil DO
          begin
            Process(chd, False);
            chd := TN.GetNextChild(chd);
          end;
      end;
    END;
  end;


begin
  TS.Clear;
  Process(TreeRoot, True);
end;

procedure TNewPickForm.LoadFromList(TS: TStrings);
// Load the tree from the passed list
VAR
  Root : TTreeNode;
  N    : Integer;

  procedure SetNone(TN : TTreeNode);
  VAR chd : TTreeNode;
  begin
    SetOneSub(TN, cSubNotP);
    SetOneFol(TN, False);
    chd := TN.GetFirstChild;
    WHILE chd <> nil DO
      begin
        SetNone(chd);
        chd := TN.GetNextChild(chd);
      end;
  end;

  procedure Match(vTN : TTreeNode; const S : String;
    stat : Integer);
  // If the string matches, set folder and subfolder according
  // to stat - 0 = both, 1 = folder only, 2 = subfolder only
  VAR
    chd : TTreeNode;
    Snu : String;
  begin
    chd := vTN.getFirstChild;
    WHILE chd <> nil DO
      begin
        IF pos(chd.text, S) = 1 THEN
          begin
            SNu := S;
            Delete(sNu, 1, Length(chd.text)+1);
            IF sNu = '' THEN
              begin
                CASE stat OF
                  0 : begin
                    SetSubAndAdjust(chd, cSubChkP);
                    SetFolAndAdjust(chd, cFolderY);
                  end;
                  1 : SetFolAndAdjust(chd, cFolderY);
                  2 : SetSubAndAdjust(chd, cSubChkP);
                END;
              end
            ELSE
              begin
                chd.Expand(False);
                Match(chd, sNu, stat);
              end;
          end;
        chd := vTN.GetNextChild(chd);
      end;
  end;


begin
  IF TS.Count = 0 THEN Exit;
  IF (TS.Count = 1) AND (Pos('Local drives', TS[0])=1) THEN Exit;
  Root := TreeRoot;
  SetNone(Root);
  Root.StateIndex := 0;
  FOR N := 0 TO TS.Count-1 DO
    Match(Root, TS[N], Integer(TS.Objects[N]));
end;


end.



